@@ -1,18 +1,18 @@
-use std::path::PathBuf;
+use std::{collections::HashSet, path::PathBuf};
 
-use anyhow::{bail, Result};
+use anyhow::Result;
 use clap::Parser;
 
 #[derive(Parser, Debug)]
 #[clap(author = "gwy15", version, about = "将 XML 弹幕转换为 ASS 文件")]
 pub struct Cli {
-    #[clap(help = "需要转换的 XML 文件")]
-    pub xml_file: PathBuf,
+    #[clap(help = "需要转换的 XML 文件或文件夹，如果是文件夹会递归将旗下所有 XML 都进行转换")]
+    pub xml_file_or_path: PathBuf,
 
     #[clap(
         long = "output",
         short = 'o',
-        help = "输出的 ASS 文件，默认为输入文件名将 .xml 替换为 .ass"
+        help = "输出的 ASS 文件，默认为输入文件名将 .xml 替换为 .ass，如果输入是文件夹则忽略"
     )]
     pub ass_file: Option<PathBuf>,
 
@@ -26,10 +26,20 @@ pub struct Cli {
         long = "font",
         short = 'f',
         help = "弹幕使用字体",
-        default_value = "sans-serif"
+        default_value = "黑体"
     )]
     font: String,
 
+    #[clap(long = "font-size", help = "弹幕字体大小", default_value = "25")]
+    font_size: u32,
+
+    #[clap(
+        long = "font-ratio",
+        help = "计算弹幕宽度时的比例，如果你的字体很宽为避免重叠需要调大这个数值",
+        default_value = "1.2"
+    )]
+    width_ratio: f64,
+
     #[clap(
         long = "duration",
         short = 'd',
@@ -61,23 +71,32 @@ pub struct Cli {
         default_value = "0.7"
     )]
     alpha: f64,
+
+    #[clap(
+        long = "force",
+        help = "默认会跳过 ass 比 xml 修改时间更晚的文件，此参数会强制转换"
+    )]
+    pub force: bool,
+
+    #[clap(
+        long = "denylist",
+        help = "黑名单，需要过滤的关键词列表文件，每行一个关键词"
+    )]
+    denylist: Option<PathBuf>,
 }
 
 impl Cli {
     pub fn check(&mut self) -> Result<()> {
-        if self.xml_file.is_dir() {
-            bail!("{} 是一个目录", self.xml_file.display());
+        if self.xml_file_or_path.is_dir() {
+            info!("输入是目录，将递归遍历目录下所有 XML 文件");
         }
-
-        if self.ass_file.is_none() {
-            let mut path = self.xml_file.clone();
-            path.set_extension("ass");
-
-            if path.is_dir() {
-                bail!("{} 是一个目录", path.display());
+        if let Some(f) = self.denylist.as_ref() {
+            if !f.exists() {
+                anyhow::bail!("黑名单文件不存在");
+            }
+            if f.is_dir() {
+                anyhow::bail!("黑名单文件不能是目录");
             }
-
-            self.ass_file = Some(path);
         }
 
         Ok(())
@@ -88,6 +107,8 @@ impl Cli {
             width: self.width,
             height: self.height,
             font: self.font.clone(),
+            font_size: self.font_size,
+            width_ratio: self.width_ratio,
             duration: self.duration,
             lane_size: self.lane_size,
             float_percentage: self.float_percentage,
@@ -95,4 +116,15 @@ impl Cli {
             bottom_percentage: 0.3,
         }
     }
+
+    pub fn denylist(&self) -> Result<Option<HashSet<String>>> {
+        match self.denylist.as_ref() {
+            None => Ok(None),
+            Some(path) => {
+                let denylist = std::fs::read_to_string(path)?;
+                let list = denylist.split('\n').map(|s| s.trim().to_string()).collect();
+                Ok(Some(list))
+            }
+        }
+    }
 }
